/*
 * spi.c - msp430f5529 launchpad spi interface implementation
 *
 * Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef SL_IF_TYPE_UART
#include <msp430f6736.h>

#include "datatypes.h"
#include "simplelink.h"
#include "spi.h"
#include "board.h"

#define ASSERT_CS()          (P4OUT &= ~BIT0)
#define DEASSERT_CS()        (P4OUT |= BIT0)

int spi_Close(Fd_t fd)
{
    /* Disable WLAN Interrupt ... */
    CC3100_InterruptDisable();

    return 0;
}

Fd_t spi_Open(char *ifName, unsigned long flags)
{
    /* Select the SPI lines: MOSI/MISO on P2.3,2 CLK on P2.5 */
    // Setup P1.0 output, P1.2 UCA0SOMI, P1.3 UCA0SIMO, P1.6 UCA0CLK
    P1OUT &= ~BIT0;                               // Clear P1.0
    P1DIR |= BIT0;                                // Set P1.0 to output direction
    P1SEL |= BIT2 | BIT3 | BIT6;                  // Set P1.0,P1.2,P1.3,P1.6 to non-IO

    // Setup eUSCI_A0
    // Setup eUSCI_A0
    UCA0CTLW0 |= UCSWRST;                         // **Put state machine in reset**
    UCA0CTLW0 |= UCMST | UCMSB | UCSYNC | UCCKPH; // 3-pin, 8-bit SPI master
                                                  // Clock polarity high, MSB
    UCA0CTLW0 |= UCSSEL_2;                        // SMCLK
    UCA0BRW_L = 0x01;                             // /2
    UCA0BRW_H = 0;                                //
    UCA0MCTLW = 0;                                // No modulation
    UCA0CTLW0 &= ~UCSWRST;                        // **Initialize USCI state machine**
    UCA0IE = UCRXIE;                              // Enable USCI_A0 RX,TX interrupt


    /* P4.1 -(HIB) WLAN enable full DS */
    P4SEL &= ~BIT1;
    P4OUT &= ~BIT1;
    P4DIR |= BIT1;


    /* Configure SPI IRQ line on P2.7 */
    P2DIR &= ~BIT7;
    P2SEL &= ~BIT7;

    P2REN |= BIT7;

    /* Configure the SPI CS to be on P4.0 */
    P4OUT |= BIT0;
    P4SEL &= ~BIT0;
    P4DIR |= BIT0;

    /* 50 ms delay */
    Delay(50);

    /* Enable WLAN interrupt */
    CC3100_InterruptEnable();

    return NONOS_RET_OK;
}


int spi_Write(Fd_t fd, unsigned char *pBuff, int len)
{
        int len_to_return = len;

    ASSERT_CS();
    while (len)
    {
        while (!(UCA0IFG&UCTXIFG));
        UCA0TXBUF = *pBuff;
        while (!(UCA0IFG&UCRXIFG));
        UCA0RXBUF;
        len --;
        pBuff++;
    }

    DEASSERT_CS();

    return len_to_return;
}


int spi_Read(Fd_t fd, unsigned char *pBuff, int len)
{
    int i = 0;

    ASSERT_CS();

    for (i = 0; i < len; i ++)
    {
        while (!(UCA0IFG&UCTXIFG));
        UCA0TXBUF = 0xFF;
        while (!(UCA0IFG&UCRXIFG));
        pBuff[i] = UCA0RXBUF;
    }

    DEASSERT_CS();

    return len;
}
#endif /* SL_IF_TYPE_UART */
